# This Makefile builds the KQML library
# 
# $Log: Makefile,v $
# Revision 1.1  1993/01/25  13:32:49  fritzson
# Initial revision
#
#

# Define relevant directories
REGEX_HOME	 = ${KQML_HOME}/regex-0.12
VPATH = $(KQML_HOME)/src/lib

KQML_INCLUDE = -I${KQML_HOME}/include -I$(KQML_ARCH)/include
KQML_LIB     = ${KQML_ARCH}/lib

CC = /usr/local/bin/gcc

#CFLAGS = -O -Wall ${INCLUDES}	# Final version
CFLAGS = -g -DDEBUG -Wall ${INCLUDES}
LDFLAGS = ${LIBDIRS} 

STDLIBS = -lsocket -lnsl -lm -lc
LOADLIBES = -lkqml ${STDLIBS}

INCLUDES = -I${KQML_HOME}/src/lib ${KQML_INCLUDE} -I${REGEX_HOME}
LIBDIRS = -L${KQML_LIB} -L/usr/local/lib

RANLIB = /usr/ccs/bin/ar ts

INSTALL = /usr/sbin/install -m 0644

KQML_SRC = init.c			\
		tcp_to_serv.c		\
		dbgmsg.c				\
		kqml_field.c		\
		kqmlmisc.c			\
		xdrkqml.c			\
		handler.c			\
		find_perf.c 		\
		prnt_kqml_msg.c	\
		send_msg.c			\
		deliver_msg.c		\
		await.c				\
		checktype.c			\
		wrapper.c			\
		kqmlparse/kqml_parser.c	\
		kqmlparse/kqml_scanner.c
SOURCES = kqmlmsg_xdr.c $(KQML_SRC)

OBJECTS = ${SOURCES:.c=.o} regex.o

# Program dependencies
all: libkqml.a

libkqml.a: ${OBJECTS}
	ar cr $@ $?
	${RANLIB} libkqml.a

regex.o:
	cd ${REGEX_HOME}; ${MAKE} -${MAKEFLAGS} regex.o
	mv $(REGEX_HOME)/regex.o .

kqmlmsg_xdr.c: kqmlmsg.x
	cp $(VPATH)/kqmlmsg.x .
	rpcgen -c -o $@ kqmlmsg.x
	rpcgen -h -o kqmlmsg.h kqmlmsg.x
	mv kqmlmsg.h ../../include

kqmlparse/kqml_parser.o:
	cd kqmlparse; ${MAKE} -${MAKEFLAGS} -f Makefile.sol

kqmlparse/kqml_scanner.o:
	cd kqmlparse; ${MAKE} -${MAKEFLAGS} -f Makefile.sol

install: all
	${INSTALL} -f ${KQML_LIB} libkqml.a
	${RANLIB} ${KQML_LIB}/libkqml.a


test: test_init test_embed test_parse

test_init: test_init.c libkqml.a
	${CC} ${CFLAGS} ${LDFLAGS} $(VPATH)/test_init.c -o $@ ${LOADLIBES}

test_embed: test_embed.c libkqml.a
	${CC} ${CFLAGS} ${LDFLAGS} $(VPATH)/test_embed.c -o $@ ${LOADLIBES}

test_parse: 
	cd kqmlparse; ${MAKE} -${MAKEFLAGS} test


clean:
	/bin/rm -f libkqml.a *.o *~ *.BAK kqmlmsg.x kqmlmsg_xdr.c test_init test_embed
	cd kqmlparse; $(MAKE) -$(MAKEFLAGS) -f Makefile.sol clean
	cd ${REGEX_HOME}; ${MAKE} -${MAKEFLAGS} clean
