/*
 * $Log: send_msg.c $
 *
 *
 */

/*
 *	(c) Copyright 1993 Paramax Systems Corp.
 *
 *	This material my be reproduced by or for the U.S.
 *	Government pursuant to the copyright license under
 *	the clause at DFARS 252.227-7013   (1991).
 */

#include <kqml/kqmlconf.h>

#include <stdarg.h>
#include <stdio.h>
#include <errno.h>
#include <malloc.h>

#include <kqml.h>
#include <kqml/kqmllib.h>

extern struct monitor_request mon_req[];

/*
 * kqml_deliver_msg 
 *
 * Deliver a message to all current SUBSCRIBErs.
 *
 *
 * Parameters:
 *
 *	1) performative
 *         The performative to be used for this KQML expression.
 *
 *	2) content
 *	   The query to be executed by the remote database.  This should
 *	   be a pointer to a character array.  The ending ';' must not be
 *	   included in the query string.
 *
 *	3) Optional (pairs of strings in the form "field", "value"
 *	   for any KQML field.
 *
 */

int
kqml_deliver_msg (char *perf, char *content, ...)
{
	char *field, **field_array, **value_array;
	va_list fp;
	int i, res, field_count;
	char end_of_arg;

	va_start (fp, content);
	field_count = 0;
	end_of_arg = FALSE;
	while (! end_of_arg)
	{
		field = va_arg (fp, char *);
		if (field != NULL)
		{
			field_count++;
			if (va_arg (fp, char *) == NULL)
			{
				end_of_arg = TRUE;
			}
		}
		else
		{
			end_of_arg = TRUE;
		}
	}
	va_end (fp);

	field_array = malloc (sizeof (char *) * (field_count + 1));
	if (field_array == NULL)
	{
		_kqml_dbg_msg (KQML_DBG_ALWAYS) 
			("kqml_deliver_msg: Could not set field array.");
		return -1;
	}
	
	value_array = malloc (sizeof (char *) * (field_count + 1));
	if (value_array == NULL)
	{
		free (field_array);
		_kqml_dbg_msg (KQML_DBG_ALWAYS) 
			("kqml_deliver_msg: Could not set value array.");
		return -1;
	}

	i = 0;
	va_start (fp, content);
	for (i = 0; i < field_count; i++)
	{
		field_array[i] = va_arg (fp, char *);
		value_array[i] = va_arg (fp, char *);
	}
	field_array[i] = NULL;
	value_array[i] = NULL;

	res = kqml_deliver_msgv (perf, content, field_array, value_array);

	free (field_array);
	free (value_array);

	return res;
}

int
kqml_deliver_msgv (char *perf, char *content, char *field[], char *value[])
{
	kqml_message *omsg;
	int res;

	omsg = kqml_build_msgv (perf, content, NULL, NULL, field, value);
	if (omsg == NULL)
	{
		return -1;
	}

	res = kqml_deliver (omsg);

	kqml_free_msg (omsg);
	return res;
}

int
kqml_deliver (kqml_message *kmsg)
{
	int i, subscriber_count, content_len, ret_value;
	char *content, **subscriber, **subscriber_id, send_success, send_error;

	send_success = send_error = FALSE;
	ret_value = 1;
	content = kqml_get_field (":content", kmsg, &content_len, NULL);

	if (content == NULL)
	{
		_kqml_dbg_msg (KQML_DBG_ALWAYS) 
			("kqml_deliver called with null content\n");
		return -1;
	}

	subscriber_count = _kqml_get_subscriber_list (content, 
		content_len, &subscriber, &subscriber_id);
	for (i = 0; i < subscriber_count; i++)
	{
		_kqml_dbg_msg (KQML_DBG_VERBOSE) ("Delivering to '%s'.", subscriber[i]);

		kqml_put_field (":receiver", kmsg, subscriber[i], -1);
		kqml_put_field (":in-reply-to", kmsg, subscriber_id[i], -1);
		if (kqml_send (0, kmsg, NULL) <= -1)
		{
			_kqml_dbg_msg (KQML_DBG_ALWAYS) ("kqml_deliver failed for '%s'\n",
				subscriber[i]);
			send_error = TRUE;
		}
		else
		{
			send_success = TRUE;
		}
	}

	if (subscriber_count)
	{
		free (subscriber);
		free (subscriber_id);
	}
	kqml_free_msg (kmsg);

	if (send_error)
	{
		return (send_success ? 0 : -1);
	}
	else
	{
		return 1;
	}
}
